import argparse

import kgd.validators as validators


def parse_args():
    """ Parse CLI arguments and return validated values """
    parser = argparse.ArgumentParser(
        description="Tool for retrieving information on tax payments by Kazakhstan companies",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("address_port", type=validators.check_ip_port,
                        help="KGD API target host and port")

    parser.add_argument("token", type=str,
                        help="KGD API secret token")

    parser.add_argument("fpath", type=validators.check_fpath,
                        help="input file with BINs")

    parser.add_argument("date_range", type=validators.check_date_range,
                        help="date range for which we retrieve data")

    parser.add_argument("-B", "--backoff", default=0.5,
                        type=validators.check_positive_float,
                        help="delay after each failed attempt")
    parser.add_argument("-w", "--timeout", default=4,
                        type=validators.check_positive_float,
                        help="server connect timeout")
    parser.add_argument("-r", "--retries",
                        default=4, type=validators.check_positive_float,
                        help="retries count for one BIN")
    parser.add_argument("-f", "--fsize",
                        default=50000000,type=validators.check_positive_int,
                        help="size limit for output file")

    return parser.parse_args()



import os
import sys

from box import Box
from tqdm import tqdm

from kgd.exceptions import NetworkError
from kgd.cli import parse_args
from kgd.constants import (PROCESSED_EXISTS_MESSAGE, PROCESSED_NOT_EXISTS_MESSAGE,
                       ExitStatus)
from kgd.parsing import TaxPaymentParser, processed_bins_fpath
from kgd.utils import load_lines, append_file, is_server_up


class TqdmUpTo(tqdm):
    def update_to(self, filename, bn, fails):
        self.set_description('Processing {} in {} ... Fails: {}'.format(bn, filename, fails))
        self.update(1)


def main():
    exit_status = ExitStatus.SUCCESS
    args = parse_args()
    p = Box(vars(args))
    prsd_bins = []

    if not is_server_up(*p.address_port.split(':')):
        return ExitStatus.ERROR_SERVER_IS_DOWN

    if not os.path.exists(processed_bins_fpath(p.fpath)):
        print(PROCESSED_NOT_EXISTS_MESSAGE)
        open(processed_bins_fpath(p.fpath), 'a').close()
    else:
        print(PROCESSED_EXISTS_MESSAGE)
        prsd_bins = load_lines(processed_bins_fpath(p.fpath))

    src_bins = load_lines(p.fpath)
    if prsd_bins:
        # TODO use s.difference_update(t)
        # bins = [x for x in src_bins if x not in prsd_bins]
        s = set(src_bins)
        s.difference_update(set(prsd_bins))
        bins = tuple(s)
    else:
        bins = tuple(src_bins)

    if not bins:
        return ExitStatus.ERROR_NO_BINS

    parser = TaxPaymentParser(p.fpath, p.fsize)
    with TqdmUpTo(total=len(bins)) as pbar:
        for bn in bins:
            # p.bn = bn
            try:
                fname = os.path.basename(parser.output_file)
                parser.process(bn, **p, reporthook=pbar.update_to(fname, bn, parser.fails))

            except NetworkError:
                # exit if it's too many fails
                fails_percent = parser.fails * 100 / len(bins)
                if fails_percent > 90:
                    return ExitStatus.ERROR
            else:
                # successfully processed BINs will be excluded next time
                append_file(processed_bins_fpath(p.fpath), bn)

    return ExitStatus.SUCCESS
